package tech.nevets.wikipediabot.commands.utils;

import net.dv8tion.jda.api.events.interaction.command.SlashCommandInteractionEvent;
import net.dv8tion.jda.api.interactions.commands.build.CommandData;
import org.jetbrains.annotations.Nullable;
import tech.nevets.wikipediabot.Bot;
import tech.nevets.wikipediabot.commands.*;

import java.util.ArrayList;
import java.util.List;

public class CommandManager {
    private static final List<ICommand> commands = new ArrayList<>();

    public CommandManager() {
        addCommand(new WikiCmd());
    }

    private void addCommand(ICommand cmd) {
        boolean nameFound = commands.stream().anyMatch(it -> it.getName().equalsIgnoreCase(cmd.getName()));
        if (nameFound) {
            throw new IllegalArgumentException("A command with this name is already present");
        }
        commands.add(cmd);
    }

    public static void registerCommands() {
        List<CommandData> cmdDataList = new ArrayList<>();
        if (commands.size() > 0) {
            commands.forEach(cmd -> cmdDataList.add(cmd.getCommandData()));
        }
        Bot.jda.updateCommands().addCommands(cmdDataList).queue();
    }

    public List<ICommand> getCommands() {
        return commands;
    }

    @Nullable
    public ICommand getCommand(String search) {
        for (ICommand cmd : commands) {
            if (cmd.getName().equalsIgnoreCase(search)) {
                return cmd;
            }
        }
        return null;
    }

    public void handle(SlashCommandInteractionEvent event) {
        String invoke = event.getName();
        ICommand cmd = getCommand(invoke);

        if (cmd != null) {
            event.deferReply().queue();
            CommandContext ctx = new CommandContext(event);
            cmd.handle(ctx);
        }
    }
}
